package com.pj.task;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.pj.api.client.level_one_server.LevelOneServerInterface;
import com.pj.bank_byte.BankByte;
import com.pj.bank_byte.BankByteMapper;
import com.pj.bank_info.BankInfo;
import com.pj.bank_info.BankInfoMapper;
import com.pj.common.core.utils.StringUtils;
import com.pj.enummj.FinishStatus;
import com.pj.rabbitmq.RabbitMQ;
import com.pj.utils.ht.AESUtil;
import org.springframework.beans.factory.BeanNameAware;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Configuration;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.transaction.annotation.Transactional;
import org.w3c.dom.Document;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;
import java.io.IOException;
import java.io.StringReader;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

@Configuration
@EnableScheduling
@Transactional(rollbackFor = Exception.class)
public class BankByteTask {

    /**
     * 银行回执字节码mapper
     */
    @Autowired
    private BankByteMapper bankByteMapper;
    /**
     * 银行回执信息mapper
     */
    @Autowired
    private BankInfoMapper bankInfoMapper;

    @Autowired
    private LevelOneServerInterface levelOneServerInterface;

    /**
     * 启动定时任务
     */
    @Scheduled(cron = "*/10 * * * * ?")  // 每10秒扫描一次
    private void bankByteTask() {

        //扫描表内是否有待处理任务
        List<BankByte> byteList = bankByteMapper.selectList(
                new LambdaQueryWrapper<BankByte>().eq(BankByte::getFinishStatus, FinishStatus.FINISH_STATUS_ZERO.getCode()).orderByAsc(BankByte::getCreateTime));
        //判断是否有任务,没有则返回
        if (byteList.size() == 0) {
            System.err.println("\n  ---  暂无待处理【银行回执信息】。" + new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(new Date()) + "\n");
            return;
        }
        //执行任务处理流程
        byteList.forEach(item -> {
            String body = item.getBody();
            //数据解析
            String xmlStr = AESUtil.decryptCBC(body, RabbitMQ.ACC_KEY);

            BankInfo bankInfo = new BankInfo();

            //银行回执——headers——获取头部字段
            String headersStr = xmlStr.substring(xmlStr.indexOf("<headers>")+9, xmlStr.indexOf("</headers>"));
            String[] headersSplit = headersStr.split("</item>");
            for (String s : headersSplit) {
                if(StringUtils.containsIgnoreCase(s,"sn")) {
                    String substring = s.substring(s.indexOf("<value>")+7, s.indexOf("</value>"));
                    bankInfo.setSn(substring);
                }
                if(StringUtils.containsIgnoreCase(s,"date")) {
                    String substring = s.substring(s.indexOf("<value>")+7, s.indexOf("</value>"));
                    bankInfo.setDate(substring);
                }
                if(StringUtils.containsIgnoreCase(s,"organization")) {
                    String substring = s.substring(s.indexOf("<value>")+7, s.indexOf("</value>"));
                    bankInfo.setOrganization(substring);
                }
                if(StringUtils.containsIgnoreCase(s,"bankCode")) {
                    String substring = s.substring(s.indexOf("<value>")+7, s.indexOf("</value>"));
                    bankInfo.setBankCode(substring);
                }
                if(StringUtils.containsIgnoreCase(s,"operation")) {
                    String substring = s.substring(s.indexOf("<value>")+7, s.indexOf("</value>"));
                    bankInfo.setOperation(substring);
                }
            }


            //银行回执——payload——获取responseDTO
            String payloadStr = xmlStr.substring(xmlStr.indexOf("</headers>")+10, xmlStr.indexOf("</message>"));
            try {
                // 创建DOM解析器
                DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
                DocumentBuilder builder = factory.newDocumentBuilder();

                // 解析XML字符串，获取Document对象
                Document document = builder.parse(new InputSource(new StringReader(payloadStr)));

                // 创建XPath对象
                XPath xpath = XPathFactory.newInstance().newXPath();

                bankInfo.setCode(xpath.evaluate("/payload/code", document));
                bankInfo.setSource(xpath.evaluate("/payload/source", document));
                bankInfo.setResult(xpath.evaluate("/payload/result", document));
                bankInfo.setNote(xpath.evaluate("/payload/note", document));

            } catch (ParserConfigurationException e) {
                e.printStackTrace();
            } catch (SAXException e) {
                e.printStackTrace();
            } catch (IOException e) {
                e.printStackTrace();
            } catch (XPathExpressionException e) {
                e.printStackTrace();
            }

            //保存银行回执信息
            int insert = bankInfoMapper.insert(bankInfo);
            //（一级市场）订单扣款-成功
            if(bankInfo.getOperation().equalsIgnoreCase("SXB011")) {
                //参数：一级市场边民订单-订单编号
                boolean result = levelOneServerInterface.confirmOrderFromBank(bankInfo.getSource());
            }
            //（一级市场）订单扣款-失败
            else if(bankInfo.getOperation().equalsIgnoreCase("SXB012")) {

            }
            //（一级市场）订单退款-成功
            else if(bankInfo.getOperation().equalsIgnoreCase("SXB014")) {

            }
            //（一级市场）订单退款-失败
            else if(bankInfo.getOperation().equalsIgnoreCase("SXB015")) {

            }

            //todo: 届时开启该数据校验
            //amqpTemplateFeign.verifyContent(dataDto);

            //数据处理完成状态
            boolean flag = false;
            // 根据类型匹配对象
            /*switch (dataDto.getDataType()) {
                case "001":
                    //对象转换
                    HtPeopleDto peopleDto = JSONObject.parseObject(body, HtPeopleDto.class);
                    //根据其内的身份证,有则新增,无则修改???
                    flag = FeignFactory.levelOneServerInterface.peopleDto(peopleDto);
                    methodHtByteTask.updateHtByteInfo(flag, item, dataDto);
                    if (!flag) {
                        System.err.println("\n边民信息处理失败。 messageId = " + item.getMessageId() + "\n");
                    } else {
                        System.err.println("\n边民信息处理成功。 messageId = " + item.getMessageId() + "\n");
                    }
                    break;
                case "002":
                    //商铺备案
                    HtShopDTO shopDto = JSONObject.parseObject(body, HtShopDTO.class);
                    //保存
                    flag = FeignFactory.levelOneServerInterface.shopDto(shopDto);
                    methodHtByteTask.updateHtByteInfo(flag, item, dataDto);
                    if (!flag) {
                        System.err.println("\n店铺信息处理失败。 messageId = " + item.getMessageId() + "\n");
                    } else {
                        System.err.println("\n店铺信息处理完成。 messageId = " + item.getMessageId() + "\n");
                    }
                    break;
                case "003":
                    //边民互助组备案
                    HtGroupDto groupDto = JSONObject.parseObject(body, HtGroupDto.class);
                    //保存
                    flag = FeignFactory.levelOneServerInterface.groupDto(groupDto);
                    methodHtByteTask.updateHtByteInfo(flag, item, dataDto);
                    if (!flag) {
                        System.err.println("\n互助组信息处理失败。 messageId = " + item.getMessageId() + "\n");
                    } else {
                        System.err.println("\n互助组信息处理完成。 messageId = " + item.getMessageId() + "\n");
                    }
                    break;
                case "004":
                    //边民合作社备案
                    HtCooperativeDto cooperativeDto = JSONObject.parseObject(body, HtCooperativeDto.class);
                    //保存
                    flag = FeignFactory.levelOneServerInterface.cooperativeDto(cooperativeDto);
                    methodHtByteTask.updateHtByteInfo(flag, item, dataDto);
                    if (!flag) {
                        System.err.println("\n边民合作社信息处理失败。 messageId = " + item.getMessageId() + "\n");
                    } else {
                        System.err.println("\n边民合作社信息处理完成。 messageId = " + item.getMessageId() + "\n");
                    }
                    break;
                case "005":
                    //商品基础信息
                    HtGoodsDto goodsDto = JSONObject.parseObject(body, HtGoodsDto.class);
                    //保存
                    flag = FeignFactory.levelOneServerInterface.goodsDto(goodsDto);
                    methodHtByteTask.updateHtByteInfo(flag, item, dataDto);
                    if (!flag) {
                        System.err.println("\n海关监管商品信息处理失败。 messageId = " + item.getMessageId() + "\n");
                    } else {
                        System.err.println("\n海关监管商品信息处理完成。 messageId = " + item.getMessageId() + "\n");
                    }
                    break;
                case "006":
                    //交易结算信息
                    HtTradeSettlement importDeclarePeople = JSONObject.parseObject(body, HtTradeSettlement.class);
                    if (importDeclarePeople == null || !importDeclarePeople.getIEFlag().toLowerCase().contains("i")) {
                        StaticLog.info("出口商品，暂时忽略:{}", body);
                        break;
                    }
                    List<BordersListDto> bordersLists = JSON.parseArray(importDeclarePeople.getBordersList(), BordersListDto.class);
                    //边民购买商品列表
                    List<BorderBuyGoodsListDto> buyGoodsList = JSON.parseArray(importDeclarePeople.getBorderBuyGoodsList(), BorderBuyGoodsListDto.class);
                    //保存航通监听记录
                    TradeSettlementDto tradeSettlementDto = new TradeSettlementDto();
                    BeanUtils.copyProperties(importDeclarePeople,tradeSettlementDto);
                    tradeSettlementDto.setBordersListDtoList(bordersLists);
                    tradeSettlementDto.setBorderBuyGoodsListDtoList(buyGoodsList);
                    importDeclarePeople.setFinishStatus(FinishStatus.FINISH_STATUS_ZERO.getCode()); // 默认待处理
                    importDeclarePeople.setCreateTime(new Date());
                    boolean b = FeignFactory.levelOneServerInterface.tradeSettlementDto(tradeSettlementDto);
                    StaticLog.info("\n交易结算信息处理完成:{}。 messageId:{} = " ,item.getMessageId() ,b);
                    methodHtByteTask.updateHtByteInfo(b, item, dataDto);
                    break;
                case "007":
                    //放行记录（过卡记录）
                    HtPassCardDTO htPassCardDTO = JSONObject.parseObject(body, HtPassCardDTO.class);
                    //系统接收到航通过卡记录且卡口类型为'3卡'：车辆已出一级市场, 修改相对应的一级市场订单状态
                    if ("3卡".equals(htPassCardDTO.getChannelType())) {
                        FeignFactory.levelOneServerInterface.orderFinish(htPassCardDTO);
                    }
                    break;
                case "008":
                    //进口申报单
                    HtImportOrderDto importOrderDto = JSONObject.parseObject(body, HtImportOrderDto.class);
                    boolean result = FeignFactory.levelOneServerInterface.importOrderDto(importOrderDto);
                    methodHtByteTask.updateHtByteInfo(result, item, dataDto);
                    if (result) {
                        System.err.println("\n进口申报单同步处理成功。 messageId = " + item.getMessageId() + "\n");
                    } else {
                        System.err.println("\n进口申报单同步处理失败。 messageId = " + item.getMessageId() + "\n");
                    }
                    break;
                case "009":
                    //进口申报单边民确认
                    HtImportDeclarePeople htImportDeclarePeople = JSONObject.parseObject(body, HtImportDeclarePeople.class);
                    htImportDeclarePeople.setFinishStatus(FinishStatus.FINISH_STATUS_ZERO.getCode());
                    htImportDeclarePeople.setCreateTime(new Date());
                    int insert2 = htImportDeclarePeopleMapper.insert(htImportDeclarePeople);
                    methodHtByteTask.updateHtByteInfo(insert2 == 1, item, dataDto);
                    if (insert2 == 1) {
                        System.err.println("\n进口申报单边民确认信息处理成功。 messageId = " + item.getMessageId() + "\n");
                    } else {
                        System.err.println("\n进口申报单边民确认信息处理失败。 messageId = " + item.getMessageId() + "\n");
                    }
                    break;
                default:
                    throw new ServiceException("类型错误。");
            }*/
        });

    }

}
