package com.pj.task;

import com.alibaba.fastjson.JSON;
import com.pj.api.dto.OrderDto;
import com.pj.api.dto.PaymentDto;
import com.pj.dto.DataDto;
import com.pj.rabbitmq.RabbitMQ;
import com.pj.utils.ht.AESUtil;
import com.pj.xml.MessageXML;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.core.AmqpTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Service
@Slf4j
public class PaymentService {

    @Autowired
    private AmqpTemplate amqpTemplate;

    /**
     * ※ 订单扣款
     * 报文名称：SXB010.xml  数据流向：一级市场--->银行
     * @param paymentDto 一级市场订单信息
     */
    public void sendSXB010(PaymentDto paymentDto) {
        log.info("send data to ht:{}", JSON.toJSONString(paymentDto));
        String xml = MessageXML.SXB010(paymentDto);
        //todo: 数据暂时定死
        DataDto dataDto = new DataDto();
        //报文加密
        String jsonString = JSON.toJSONString(xml);
        String encryptCBC = AESUtil.encryptCBC(jsonString, RabbitMQ.ACC_KEY);
        dataDto.setData(encryptCBC);

        //mq发送信息
        amqpTemplate.convertAndSend(RabbitMQ.TEST_SEND_QUEUE, dataDto);
        log.info("send mq msg finish:{}", JSON.toJSONString(xml));
    }


    /**
     * ※ 订单退款
     * 报文名称：SXB013.xml  数据流向：一级市场--->银行
     * @param paymentDto 一级市场订单信息
     */
    public void sendSXB013(PaymentDto paymentDto) {
        log.info("send data to ht:{}", JSON.toJSONString(paymentDto));
        String xml = MessageXML.SXB013(paymentDto);
        //todo: 数据暂时定死
        DataDto dataDto = new DataDto();
        //报文加密
        String jsonString = JSON.toJSONString(xml);
        String encryptCBC = AESUtil.encryptCBC(jsonString, RabbitMQ.ACC_KEY);
        dataDto.setData(encryptCBC);

        //mq发送信息
        amqpTemplate.convertAndSend(RabbitMQ.TEST_SEND_QUEUE, dataDto);
        log.info("send mq msg finish:{}", JSON.toJSONString(xml));
    }
    /**
     * 获取航通数据时,验证其是否被篡改
     */
    /*public boolean verifyContent(DataDto dataDto){
        //获取其签名
        String sign = dataDto.getSign();
        String signString = "msgId=" + dataDto.getMsgId() + "&appId=" + dataDto.getAppId() + "&dataType=" + dataDto.getDataType() + "&msgTime=" + dataDto.getMsgTime() + "&data=" + dataDto.getData();
        String md5 = MD5.getMd5(signString, 32);
        if(!sign.equals(md5))
            throw new ServiceException("报文已经被修改。");
        return true;
    }*/


}
